@@ -3,7 +3,7 @@
 #include "gnome.h"
 
 int hide_game_over_box_callback (GtkWidget *app, void *data );
-int hide_select_box_callback (GtkWidget *app, void *data );
+void hide_select_box_callback (GtkWidget *app, void *data );
 void hide_game_over_box();
 void hide_select_box();
 void show_game_over_dialog(gboolean);
