@@ -1,4 +1,5 @@
-/* io-gtk.c
+/* io-gtk.c --
+   Copyright (C) 1998 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and'or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,10 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+   USA */
 
-/* Written by Changwoo Ryu <cwryu@eve.kaist.ac.kr>. */
+/* Written by Changwoo Ryu <cwryu@adam.kaist.ac.kr>. */
 
 #include <config.h>
 
@@ -28,7 +30,6 @@
 
 
 
-#include "freecell.h"
 #include "card.h"
 #include "game.h"
 
@@ -114,7 +115,7 @@ io_gtk_init (void)
 
   
   /* make main window.  */
-  main_window = gnome_app_new ("freecell", "FreeCell");
+  main_window = gnome_app_new ("freecell", _("Freecell"));
   gtk_signal_connect (GTK_OBJECT(main_window), "delete_event",
 		      GTK_SIGNAL_FUNC(callback_exit), NULL);
   gtk_widget_realize(main_window);
@@ -172,11 +173,11 @@ io_gtk_init (void)
 				   destination_drawing_areas[i]);
       gtk_widget_show(destination_drawing_areas[i]);
     }
-  gtk_box_pack_start_defaults (GTK_BOX(vbox), hbox);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);
   
   w = gtk_hseparator_new();
-  gtk_box_pack_start_defaults (GTK_BOX(vbox), w);
+  gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
   gtk_widget_show (w);
   
   hbox = gtk_hbox_new (FALSE, 2);
@@ -312,11 +313,13 @@ callback_restart (GtkWidget *widget, GdkEvent *event)
       && freecellgame
       && !freecellgame_is_finished(freecellgame))
     {
-      mb = gnome_messagebox_new (_("Exit this game?"),
-				 GNOME_MESSAGEBOX_QUESTION,
-				 _("Yes"), _("No"), NULL);
+      mb = gnome_message_box_new (_("Exit this game?"),
+				  GNOME_MESSAGE_BOX_QUESTION,
+				  GNOME_STOCK_BUTTON_YES,
+				  GNOME_STOCK_BUTTON_NO,
+				  NULL);
       GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-      gnome_messagebox_set_modal (GNOME_MESSAGEBOX(mb));
+      gnome_dialog_set_modal (GNOME_DIALOG(mb));
       gtk_signal_connect_object(GTK_OBJECT(mb),
 				"clicked",
 				GTK_SIGNAL_FUNC(callback_restart_lose),
@@ -331,7 +334,7 @@ callback_restart (GtkWidget *widget, GdkEvent *event)
 static void
 callback_restart_lose (GtkWidget *widget, gpointer data)
 {
-  if ((int)data == 1)
+  if ((int)data == 0)
     {
       score_add_lose ();
       callback_restart_really ();
@@ -366,6 +369,7 @@ static void
 callback_seed_input (GtkWidget *widget, gpointer data)
 {
   int seed;
+  char buffer[64];
 
   seed = atoi (GTK_ENTRY(data)->text);
   
@@ -373,6 +377,9 @@ callback_seed_input (GtkWidget *widget, gpointer data)
     freecellgame_delete (freecellgame);
 
   freecellgame = freecellgame_new_with_seed (4, 8, seed);
+  sprintf (buffer, _("Freecell #%d"), freecellgame->seed);
+  gtk_window_set_title (GTK_WINDOW (main_window), buffer);
+  
 
   stalled = 0;
   selected = SELECTED_NONE;
@@ -386,45 +393,24 @@ callback_seed_input (GtkWidget *widget, gpointer data)
 static void
 callback_new_with_seed_really (void)
 {
-  GtkWidget *dialog, *entry, *label, *button, *vbox;
+  GtkWidget *dialog, *entry, *label;
+
+  dialog = gnome_dialog_new (_("Seed"), _("OK"), _("Cancel"), NULL);
 
-  dialog = gtk_dialog_new ();
-  GTK_WINDOW(dialog)->position = GTK_WIN_POS_MOUSE;
-  gtk_window_set_title (GTK_WINDOW(dialog), _("Seed"));
-  gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
-                      GTK_SIGNAL_FUNC (callback_cancel),
-                      (gpointer)dialog);
   label = gtk_label_new (_("Enter the seed."));
-  gtk_box_pack_start_defaults (GTK_BOX(GTK_DIALOG(dialog)->vbox), label);
+  gtk_box_pack_start_defaults (GTK_BOX(GNOME_DIALOG(dialog)->vbox), label);
   gtk_widget_show (label);
   
   entry = gtk_entry_new ();
-  gtk_box_pack_start_defaults (GTK_BOX(GTK_DIALOG(dialog)->vbox), entry);
+  gtk_box_pack_start_defaults (GTK_BOX(GNOME_DIALOG(dialog)->vbox), entry);
   gtk_widget_show (entry);
 
-  vbox = gtk_hbox_new (TRUE, 4);
-  gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), vbox);
-  
-  button = gtk_button_new_with_label (_("OK"));
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-                      GTK_SIGNAL_FUNC (callback_seed_input),
-                      (gpointer)entry);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-                      GTK_SIGNAL_FUNC (callback_cancel),
-                      (gpointer)dialog);
-  gtk_box_pack_start_defaults (GTK_BOX(vbox), button);
-  gtk_widget_show(button);
-  
-  button = gtk_button_new_with_label (_("Cancel"));
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-                      GTK_SIGNAL_FUNC (gtk_widget_destroy),
-                      (gpointer)dialog);
-  gtk_box_pack_start_defaults (GTK_BOX(vbox), button);
-  gtk_widget_show(button);
-  
-  gtk_widget_show(vbox);
-  
-  gtk_grab_add (dialog);
+  gnome_dialog_button_connect (GNOME_DIALOG(dialog), 0,
+			       GTK_SIGNAL_FUNC (callback_seed_input), entry);
+  gnome_dialog_button_connect (GNOME_DIALOG(dialog), 0,
+			       GTK_SIGNAL_FUNC (callback_cancel), dialog);
+  gnome_dialog_button_connect (GNOME_DIALOG(dialog), 1,
+			       GTK_SIGNAL_FUNC (callback_cancel), dialog);
   gtk_widget_show (dialog);
 }
 
@@ -432,7 +418,7 @@ callback_new_with_seed_really (void)
 static void
 callback_new_with_seed_with_lose (GtkWidget *widget, gpointer data)
 {
-  if ((int)data == 1)
+  if ((int)data == 0)
     {
       score_add_lose();
       callback_new_with_seed_really();
@@ -449,11 +435,13 @@ callback_new_with_seed (GtkWidget *widget, GdkEvent *event)
       && freecellgame
       && !freecellgame_is_finished(freecellgame))
     {
-      mb = gnome_messagebox_new (_("Exit this game?"),
-				 GNOME_MESSAGEBOX_QUESTION,
-				 _("Yes"), _("No"), NULL);
+      mb = gnome_message_box_new (_("Exit this game?"),
+				  GNOME_MESSAGE_BOX_QUESTION,
+				  GNOME_STOCK_BUTTON_YES,
+				  GNOME_STOCK_BUTTON_NO,
+				  NULL);
       GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-      gnome_messagebox_set_modal (GNOME_MESSAGEBOX(mb));
+      gnome_dialog_set_modal (GNOME_DIALOG(mb));
       gtk_signal_connect_object(GTK_OBJECT(mb),
 				"clicked",
 				GTK_SIGNAL_FUNC(callback_new_with_seed_with_lose),
@@ -467,16 +455,21 @@ callback_new_with_seed (GtkWidget *widget, GdkEvent *event)
 static void
 callback_new_really_callback (GtkWidget *widget, gpointer data)
 {
-	callback_new_really ();
+  if ((int)data == 0)
+    callback_new_really ();
 }
 
 static void
 callback_new_really (void)
 {
+  char buffer[64];
+  
   if (freecellgame)
     freecellgame_delete (freecellgame);
   
   freecellgame = freecellgame_new (4, 8);
+  sprintf (buffer, _("Freecell #%d"), freecellgame->seed);
+  gtk_window_set_title (GTK_WINDOW (main_window), buffer);
 
   stalled = 0;
   selected = SELECTED_NONE;
@@ -490,7 +483,7 @@ callback_new_really (void)
 static void
 callback_new_with_lose (GtkWidget *widget, gpointer data)
 {
-  if ((int)data == 1)
+  if ((int)data == 0)
     {
       score_add_lose();
       callback_new_really();
@@ -507,11 +500,13 @@ callback_new (GtkWidget *widget, GdkEvent *event)
       && freecellgame
       && !freecellgame_is_finished(freecellgame))
     {
-      mb = gnome_messagebox_new (_("Exit this game?"),
-				 GNOME_MESSAGEBOX_QUESTION,
-				 _("Yes"), _("No"), NULL);
+      mb = gnome_message_box_new (_("Exit this game?"),
+				  GNOME_MESSAGE_BOX_QUESTION,
+				  GNOME_STOCK_BUTTON_YES,
+				  GNOME_STOCK_BUTTON_NO,
+				  NULL);
       GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-      gnome_messagebox_set_modal (GNOME_MESSAGEBOX(mb));
+      gnome_dialog_set_modal (GNOME_DIALOG(mb));
       gtk_signal_connect_object(GTK_OBJECT(mb),
 				"clicked",
 				GTK_SIGNAL_FUNC(callback_new_with_lose),
@@ -534,14 +529,12 @@ callback_score (GtkWidget *widget, GdkEvent *event)
 void
 callback_undo (GtkWidget *widget, GdkEvent *event)
 {
-  GtkWidget *mb;
-
-  mb = gnome_messagebox_new (_("Sorry, this feature is not implemented (yet)."),
-			     GNOME_MESSAGEBOX_INFO,
-			     _("OK"), NULL, NULL);
-  GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-  gnome_messagebox_set_modal (GNOME_MESSAGEBOX(mb));
-  gtk_widget_show (mb);
+  if (freecellgame && freecellgame_undo (freecellgame) > 0)
+    {
+      selected = SELECTED_NONE;
+      inverted = 0;
+      refresh_all ();
+    }
 }
 
 
@@ -551,6 +544,8 @@ callback_option (GtkWidget *widget, GdkEvent *event)
   GtkWidget *dialog;
 
   dialog = option_dialog ();
+
+  gtk_widget_show (dialog);
 }
 
 
@@ -559,11 +554,12 @@ callback_rule (GtkWidget *widget, GdkEvent *event)
 {
   GtkWidget *mb;
 
-  mb = gnome_messagebox_new (_("Sorry, this feature is not implemented (yet)."),
-			     GNOME_MESSAGEBOX_INFO,
-			     _("OK"), NULL, NULL);
+  mb = gnome_message_box_new (_("Sorry, this feature is not implemented (yet)."),
+			      GNOME_MESSAGE_BOX_INFO,
+			      GNOME_STOCK_BUTTON_OK,
+			      NULL);
   GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-  gnome_messagebox_set_modal (GNOME_MESSAGEBOX(mb));
+  gnome_dialog_set_modal (GNOME_DIALOG(mb));
   gtk_widget_show (mb);
 }
 
@@ -579,7 +575,7 @@ callback_exit_really (void)
 static void
 callback_exit_with_lose (GtkWidget *widget, gpointer data)
 {
-  if ((int)data == 1)
+  if ((int)data == 0)
     {
       score_add_lose();
       callback_exit_really();
@@ -596,11 +592,14 @@ callback_exit (GtkWidget *widget, GdkEvent *event)
       && freecellgame
       && !freecellgame_is_finished(freecellgame))
     {
-      mb = gnome_messagebox_new (_("Exit this game?"),
-				 GNOME_MESSAGEBOX_QUESTION,
-				 _("Yes"), _("No"), NULL);
+      mb = gnome_message_box_new (_("Exit this game?"),
+				  GNOME_MESSAGE_BOX_QUESTION,
+				  GNOME_STOCK_BUTTON_YES,
+				  GNOME_STOCK_BUTTON_NO,
+				  NULL);
+
       GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-      gnome_messagebox_set_modal (GNOME_MESSAGEBOX(mb));
+      gnome_dialog_set_modal (GNOME_DIALOG(mb));
       gtk_signal_connect_object(GTK_OBJECT(mb),
 				"clicked",
 				GTK_SIGNAL_FUNC(callback_exit_with_lose),
@@ -619,19 +618,21 @@ callback_about (GtkWidget *widget, GdkEvent *event)
     N_("Changwoo Ryu."),
     NULL
   };
+
+#ifdef ENABLE_NLS
 #ifndef ELEMENTS  
 #define ELEMENTS(x) (sizeof(x) / sizeof(x[0]))
-#endif
-
+#endif /* ELEMENTS */
   {
     int i;
 
     for (i = 0; i < (ELEMENTS(authors) - 1); i++)
       authors[i] = _(authors[i]);
   }  
+#endif /* ENABLE_NLS */
   
-  about = gnome_about_new (_("FreeCell"), VERSION,
-			   _("(C) 1997-1998 Changwoo Ryu"),
+  about = gnome_about_new (_("Freecell"), VERSION,
+			   "(C) 1998 Free Software Foundation, Inc.",
 			   authors,
 			   _("Reimplement the popular solitaire card game."),
 			   NULL);
@@ -643,11 +644,12 @@ inform_invalid_move (void)
 {
   GtkWidget *mb;
 
-  mb = gnome_messagebox_new (_("You can't move in that way."),
-			     GNOME_MESSAGEBOX_ERROR,
-			     _("OK"), NULL, NULL);
+  mb = gnome_message_box_new (_("You can't move in that way."),
+			      GNOME_MESSAGE_BOX_ERROR,
+			      GNOME_STOCK_BUTTON_OK,
+			      NULL);
   GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-  gnome_messagebox_set_modal (GNOME_MESSAGEBOX(mb));
+  gnome_dialog_set_modal (GNOME_DIALOG(mb));
   gtk_widget_show (mb);
 }
 
@@ -874,23 +876,30 @@ callback_field_press (GtkWidget *widget, GdkEventButton *event,
 		  selected = SELECTED_NONE;
 		  refresh_field(selected_index);
 		}
-	      else if (freecellgame_field_to_field_sequence(freecellgame,
-							    selected_index,
-							    index) < 0)
-		{
-		  if (option_inform_invalid_move)
-		    {
-		      inform_invalid_move();
-		      selected = SELECTED_NONE;
-		      refresh_field(selected_index);
-		    }
-		}
 	      else
 		{
-		  selected = SELECTED_NONE;
-		  refresh_field(selected_index);
-		  refresh_field(index);
-		  to_destination_auto();
+		  if (option_move_one_by_one)
+		    tmp = freecellgame_field_to_field (freecellgame,
+						       selected_index,
+						       index);
+		  else
+		    tmp = freecellgame_field_to_field_sequence (freecellgame,
+							        selected_index,
+							        index);
+
+		  if ((tmp < 0) && option_inform_invalid_move)
+		    {
+			  inform_invalid_move();
+			  selected = SELECTED_NONE;
+			  refresh_field (selected_index);
+		    }
+		  else
+		    {
+		      selected = SELECTED_NONE;
+		      refresh_field (selected_index);
+		      refresh_field (index);
+		      to_destination_auto ();
+		    }
 		}
 	    }
 	  else
@@ -988,7 +997,7 @@ callback_field_release (GtkWidget *widget, GdkEventButton *event,
 static void
 update_cursors (void)
 {
-  int i;
+  int i, tmp;
 
   switch (selected)
     {
@@ -1026,9 +1035,12 @@ update_cursors (void)
 	}
       for (i = 0; i < 8; i++)
 	{
-	  if (freecellgame_can_move_field_to_field_sequence (freecellgame,
-							     selected_index,
-							     i))
+	  if (option_move_one_by_one)
+	    tmp = freecellgame_can_move_field_to_field (freecellgame, selected_index, i);
+	  else
+	    tmp = freecellgame_can_move_field_to_field_sequence (freecellgame, selected_index, i);
+
+	  if (tmp)
 	    gdk_window_set_cursor(field_drawing_areas[i]->window,
 				  up_cursor);
 	  else
@@ -1119,11 +1131,13 @@ to_destination_auto(void)
 
   if (freecellgame_is_there_no_way(freecellgame))
     {
-      mb = gnome_messagebox_new (_("Sorry, there is no card to move."),
-				 GNOME_MESSAGEBOX_INFO,
-				 _("OK"), NULL, NULL);
+      mb = gnome_message_box_new (_("Sorry, there is no card to move."),
+				  GNOME_MESSAGE_BOX_INFO,
+				  GNOME_STOCK_BUTTON_OK,
+				  NULL);
+
       GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-      gnome_messagebox_set_modal (GNOME_MESSAGEBOX(mb));
+      gnome_dialog_set_modal (GNOME_DIALOG(mb));
       gtk_widget_show (mb);
       score_add_lose();
       stalled = 1;
@@ -1131,11 +1145,13 @@ to_destination_auto(void)
   
   if (freecellgame_is_finished(freecellgame))
     {
-      mb = gnome_messagebox_new (_("Congraturations.  You won.\nDo you want to play again?"),
-				 GNOME_MESSAGEBOX_INFO,
-				 _("Yes"), _("No"), NULL, NULL);
+      mb = gnome_message_box_new (_("Congraturations.  You won.\nDo you want to play again?"),
+				  GNOME_MESSAGE_BOX_QUESTION,
+				  GNOME_STOCK_BUTTON_YES,
+				  GNOME_STOCK_BUTTON_NO,
+				  NULL);
       GTK_WINDOW(mb)->position = GTK_WIN_POS_MOUSE;
-      gnome_messagebox_set_modal (GNOME_MESSAGEBOX(mb));
+      gnome_dialog_set_modal (GNOME_DIALOG(mb));
       gtk_signal_connect_object (GTK_OBJECT (mb),
 				 "clicked",
 				 GTK_SIGNAL_FUNC(callback_new_really_callback),
