@@ -69,6 +69,7 @@ char *fsize2names[] = {
 	N_("Tiny"),
 	N_("Medium"),
 	N_("Biiiig"),
+	N_("Custom"),
 };
 
 void show_face(GtkWidget *pm)
@@ -113,10 +114,8 @@ void top_ten(GtkWidget *widget, gpointer data)
 {
 	gchar buf[64];
 
-	if(fsize<3)
+	if(fsize<4)
 		strncpy(buf, fsize2names[fsize], sizeof(buf));
-	else
-		g_snprintf(buf, sizeof(buf), "%dx%dx%d",xsize,ysize,nmines);
 
 	show_scores(buf, 0);
 }
@@ -157,18 +156,24 @@ void win_game(GtkWidget *widget, gpointer data)
         int pos;
         gchar buf[64];
 
-        show_face(pm_win);
         gtk_clock_stop(GTK_CLOCK(clk));
+        show_face(pm_win);
 
-	score = (gfloat) (GTK_CLOCK(clk)->stopped / 60) + 
-		(gfloat) (GTK_CLOCK(clk)->stopped % 60) / 100; 
+	if(fsize<3) {
+	    score = (gfloat) (GTK_CLOCK(clk)->stopped / 60) + 
+		    (gfloat) (GTK_CLOCK(clk)->stopped % 60) / 100;
 
-	if(fsize<3)
-		strncpy(buf, fsize2names[fsize], sizeof(buf));
-	else
-		g_snprintf(buf, sizeof(buf), "%dx%dx%d",xsize,ysize,nmines);
+            strncpy(buf, fsize2names[fsize], sizeof(buf));
+	    pos = gnome_score_log(score, buf, FALSE);
+
+	} else {
+	    score = ((nmines * 100) / (xsize * ysize)) /
+		    (gfloat) (GTK_CLOCK(clk)->stopped); 
+
+            strncpy(buf, fsize2names[fsize], sizeof(buf));
+	    pos = gnome_score_log(score, buf, TRUE);
+	}
 
-	pos = gnome_score_log(score, buf, FALSE);
 	show_scores(buf, pos);
 }
 
@@ -184,7 +189,7 @@ void unlook_cell(GtkWidget *widget, gpointer data)
 
 void setup_mode(GtkWidget *widget, gint mode)
 {
-	gint size_table[3][3] = {{ 10, 10, 10 }, {20, 20, 50}, {35, 35, 170}};
+	gint size_table[3][3] = {{ 8, 8, 10 }, {16, 16, 40}, {30, 16, 99}};
 	gint x,y,m,s;
 
 	if (mode == 3) {
@@ -201,6 +206,23 @@ void setup_mode(GtkWidget *widget, gint mode)
 	gtk_minefield_set_mines(GTK_MINEFIELD(mfield), m, s);
 }
 
+int range (int val, int min, int max)
+{
+	if (val < min)
+		val = min;
+	if (val > max)
+		val = max;
+	return val;
+}
+
+void verify_ranges (void)
+{
+	minesize = range (minesize, 2, 100);
+	xsize    = range (xsize, 4, 100);
+	ysize    = range (xsize, 4, 100);
+	nmines   = range (nmines, 1, xsize * ysize);
+}
+
 void do_setup(GtkWidget *widget, gpointer data)
 {
         guint oldxsize, oldysize, oldnmines, oldfsize;
@@ -215,7 +237,8 @@ void do_setup(GtkWidget *widget, gpointer data)
         nmines = atoi(gtk_entry_get_text(GTK_ENTRY(mentry)));
         minesize = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(sentry));
 	fsize  = fsc;
-	
+
+	verify_ranges ();
         setup_mode(mfield, fsize);
   
         if ((oldxsize != xsize) ||
@@ -524,57 +547,60 @@ static int xpos, ypos;
 static error_t
 parse_an_arg (int key, char *arg, struct argp_state *state)
 {
-  switch (key)
-    {
-    case 'x':
-      x_set = 1;
-      xsize = atoi (arg);
-      break;
-    case 'y':
-      y_set = 1;
-      ysize = atoi (arg);
-      break;
-    case 's':
-      minesize_set = 1;
-      minesize = atoi (arg);
-      break;
-    case 'n':
-      nmines_set = 1;
-      nmines = atoi (arg);
-      break;
-    case 'f':
-      fsize_set = 1;
-      fsize = atoi (arg);
-      break;
-    case 'a':
-      set_pos |= 1;
-      xpos = atoi (arg);
-      break;
-    case 'b':
-      set_pos |= 2;
-      ypos = atoi (arg);
-      break;
-    case ARGP_KEY_SUCCESS:
-      if (set_pos == 3)
-	gtk_widget_set_uposition (window, xpos, ypos);
-
-      if (! x_set)
-	xsize  = gnome_config_get_int("/gnomine/geometry/xsize=20");
-      if (! y_set)
-	ysize  = gnome_config_get_int("/gnomine/geometry/ysize=20");
-      if (! nmines_set)
-	nmines = gnome_config_get_int("/gnomine/geometry/nmines=50");
-      if (! minesize_set)
-	minesize = gnome_config_get_int("/gnomine/geometry/minesize=17");
-      if (! fsize_set)
-	fsize  = gnome_config_get_int("/gnomine/geometry/mode=0");
-      break;
-
-    default:
-      return ARGP_ERR_UNKNOWN;
-    }
-
-  return 0;
+	switch (key)
+	{
+	case 'x':
+		x_set = 1;
+		xsize = atoi (arg);
+		break;
+	case 'y':
+		y_set = 1;
+		ysize = atoi (arg);
+		break;
+	case 's':
+		minesize_set = 1;
+		minesize = atoi (arg);
+		break;
+	case 'n':
+		nmines_set = 1;
+		nmines = atoi (arg);
+		break;
+	case 'f':
+		fsize_set = 1;
+		fsize = atoi (arg);
+		break;
+	case 'a':
+		set_pos |= 1;
+		xpos = atoi (arg);
+		break;
+	case 'b':
+		set_pos |= 2;
+		ypos = atoi (arg);
+		break;
+	case ARGP_KEY_SUCCESS:
+		if (set_pos == 3)
+			gtk_widget_set_uposition (window, xpos, ypos);
+		
+		if (! x_set)
+			xsize  = gnome_config_get_int("/gnomine/geometry/xsize=16");
+		if (! y_set)
+			ysize  = gnome_config_get_int("/gnomine/geometry/ysize=16");
+		if (! nmines_set)
+			nmines = gnome_config_get_int("/gnomine/geometry/nmines=40");
+		if (! minesize_set){
+			minesize = gnome_config_get_int("/gnomine/geometry/minesize=17");
+			if (minesize < 0)
+				minesize = 1;
+		}
+		if (! fsize_set)
+			fsize  = gnome_config_get_int("/gnomine/geometry/mode=0");
+		break;
+		
+	default:
+		return ARGP_ERR_UNKNOWN;
+	}
+	
+	return 0;
 }
 
 int
@@ -598,6 +624,7 @@ main (int argc, char *argv[])
 
         gnome_init("gnomine", &parser, argc, argv, 0, NULL);
 
+	verify_ranges ();
         gdk_imlib_init ();
 
 #ifdef ENABLE_NLS 
